@@ -129,7 +129,9 @@ U16 CInputScript::GetTag(void)
 void CInputScript::GetRect (Rect * r)
 {
     InitBits();
+
     int nBits = (int) GetBits(5);
+
     r->xmin = GetSBits(nBits);
     r->xmax = GetSBits(nBits);
     r->ymin = GetSBits(nBits);
@@ -1920,6 +1922,11 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 {
     int status = FLASH_PARSE_ERROR;
 
+    int z_status;
+    uLongf z_outsize;
+
+    char *z_data, *tmp_data;
+
     m_fileBuf = (unsigned char *)data;
     m_actualSize = size;
 
@@ -1930,14 +1937,13 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 		return FLASH_PARSE_NEED_DATA;	// No, need more data
 	    }
 
-	    needHeader = 0;	// Yes
-
 	    U8 fileHdr[8];
 
 	    memcpy(fileHdr,data,8);
 
 	    // Verify the header and get the file size.
-	    if (fileHdr[0] != 'F' || fileHdr[1] != 'W' || fileHdr[2] != 'S' )
+	    if (((fileHdr[0] != 'F') && (fileHdr[0] != 'C')) 
+	    	|| fileHdr[1] != 'W' || fileHdr[2] != 'S' )
 	    {
 		//fprintf(stderr, "Not a Flash File.\n");
 		return FLASH_PARSE_ERROR;	// Error
@@ -1954,6 +1960,7 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 		      | ((U32) fileHdr[6] << 16)
 		      | ((U32) fileHdr[7] << 24);
 
+
 	    // Verify the minimum length of a Flash file.
 	    if (m_fileSize < 21)
 	    {
@@ -1961,9 +1968,46 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 		return FLASH_PARSE_ERROR;	// Error
 	    }
 
+	    // Decompress the file if it is compressed
+	    if (fileHdr[0] == 'C') {
+		// Copy the compressed data to a tmp-container
+		tmp_data=(char*)malloc(size);
+		memcpy(tmp_data, data, size);
+
+		// Make the output-buffer ready
+		z_data=(char*)malloc(m_fileSize-8);
+		z_outsize=m_fileSize-8;
+
+		// Uncompress using zlib
+		z_status = uncompress((Bytef*)z_data, &z_outsize,(Bytef*) &tmp_data[8], size-8);
+
+		// Check for error, if any it must be because 
+		// the file/data hasen't been loaded fully - i hope
+		if (z_status < 0) {
+			free(tmp_data);
+			free(z_data);
+			return FLASH_PARSE_NEED_DATA;
+		}
+		// Clean up and move the header of the file
+		// plus the decompressed data to a new container
+		free(tmp_data);
+		data=(char*)realloc(data, m_fileSize);
+		memcpy(data, fileHdr, 8);
+		memcpy(&data[8],z_data, (size_t)z_outsize);
+		free(z_data);
+
+		// Update some 'global' variables
+		m_fileBuf = (unsigned char *)data;
+		m_actualSize = m_fileSize;
+
+	    }
 	    // Set the file position past the header and size information.
 	    m_filePos = 8;
 
+	    // The header is no longer needed size the info has been extracted,
+	    // and at this point there is no danger of a "return FLASH_PARSE_NEED_DATA"
+	    needHeader = 0;	// Yes
+
 	    // Get the frame information.
 	    GetRect(&frameRect);
 
@@ -1981,7 +2025,6 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
     }
 
     ParseTags(&status);
-
     return status;
 }
 
